/** Runner.java

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

 This work is based on Dr. Joao Jose Neto theories and techniques on
 Automata and Formal Languages (http://www.pcs.usp.br/~lta).

*/
package adaptools.vm;

import java.io.File;
import java.util.StringTokenizer;


/**

 Command line based interface to virtual machine.
 HINT: Look at function main to learn a simple way to call a virtual machine
 from another program.

 @author Hemerson Pistori (pistori@ec.ucdb.br)

*/
public class Runner {

	private String projectDirName = "";
	private String projectCompleteName = "";
	private String objectFileName = null;
	private String semanticAction = null;
	private String inputFileName = null;
	private String outputFileName = null;
	private String transducerOutputFileName = null;
	private String inputMachineFileName = null;
	private int debugLevel = 1;

    private void loadProject(String projectFileName) {

    	System.out.println("em loadP = "+projectFileName);

         projectDirName = (new File(projectFileName)).getParent();
         projectCompleteName = (new File(projectFileName)).getPath();
         projectFileName = (new File(projectFileName)).getName();

         String str = Utils.fileToString(projectDirName,projectFileName);
         String value = null;
         String label = null;


         if(str != null && str.length() > 0) { 
            StringTokenizer tokens = new StringTokenizer(str,";\n");
            while( tokens.hasMoreTokens() ) {
				label = tokens.nextToken();
				if(tokens.hasMoreTokens()) {
				  value = tokens.nextToken();
				  if( label.equals("[Object]")) objectFileName = value;
                  if( label.equals("[Input]"))  inputFileName = value;
                  if( label.equals("[Output]")) outputFileName = value;
                  if( label.equals("[SemanticAction]")) semanticAction = value;
                  if( label.equals("[InputMachine]")) inputMachineFileName = value;
			    }
		    }
         }

    }

    // Extract parameters from command line.
    private boolean parseCommandLine(String args[]) {

		if(args.length < 1) return(false);

        for( int i = 0 ; i < args.length; ++i ) {
			if(args[i].equals("-h") || args[i].equals("-help") ) return(false);
			else if(args[i].equals("-m")) objectFileName = args[i+1];
			else if(args[i].equals("-s")) semanticAction = args[i+1];
			else if(args[i].equals("-i")) inputFileName = args[i+1];
			else if(args[i].equals("-o")) outputFileName = args[i+1];
			else if(args[i].equals("-t")) transducerOutputFileName = args[i+1];
			else if(args[i].equals("-im")) inputMachineFileName = args[i+1];
			else if(args[i].equals("-p")) loadProject(args[i+1]);
			else if(args[i].equals("-d")) {
				if( ((i+1) >= args.length) || args[i+1].substring(0,1).equals("-"))
				   debugLevel = 1;
				else debugLevel = Integer.parseInt(args[i+1]);
		    }
	    }

        if(outputFileName == null) outputFileName = "output.tmp";
        if(transducerOutputFileName == null) transducerOutputFileName = "outputTrans.tmp";

		return( objectFileName != null && inputFileName != null);
    }

    private void printHelp() {
           System.out.println(
			  "Usage: java adaptools.vm.Runner <options>\n"+
			  "where options are:\n"+
			  "  -m machine_code_file (required)\n"+
			  "  -s semantic action\n"+
			  "  -i input_file (required)\n"+
			  "  -o output_file (required)\n"+
			  "  -p project_file (replaces -m -i and -o)\n"+
			  "  -t transducer_output_file (optional)\n"+
			  "  -im input_machine (optional)\n"+
			  "  -d [level] (optional) //Debug Information Level - Varies from 0 to 9 - Default=1]\n"+
              "\nExample 1: java adaptools.vm.Runner -m cc//sintatico.spa -i cc//lista.in -o lista.spa -im ccl//lexico.spa\n"+
              "\nExample 2: java adaptools.vm.Runner -p cc//sintatico.prj\n"
		   );
    }

    public void run(String args[],Vmds caller) {
        if(parseCommandLine(args) == false) {
			printHelp();
	    } else {

           System.out.print(
			  "\nMachine  = "+objectFileName+
			  "\nInput    = "+inputFileName+			  
			  "\nOutput   = "+outputFileName+
			  "\nTransducer Output  = "+transducerOutputFileName+
			  "\nConnection (Input) = "+inputMachineFileName+"\n"+
			  "\nSemantic Action = "+semanticAction+"\n"+
			  "\nExecuting..."
		   );


           VmdsImpl vmds = new VmdsImpl();
           vmds.setLastVisitedDir(projectCompleteName);
           vmds.setObjectFileName(objectFileName);
           vmds.setInputFileName(inputFileName);
           vmds.setOutputFileName(outputFileName);
           vmds.setTransducerOutputFileName(transducerOutputFileName);
           vmds.setDebugLevel(debugLevel);
           vmds.setSemanticAction(semanticAction);
           if(inputMachineFileName != null) {
               System.out.println("Vai conectar");
               vmds.connectInput(vmds.getCurrentDir(),inputMachineFileName);
               vmds.getInputMachine().setDebugLevel(debugLevel);
           }
           vmds.run(caller);
	    }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {

    	String[] arg2 = new String[4];
    	for(int i = 0; i < args.length; i++){
    		arg2[i] = new String();
    	}
    	arg2[0] = "-m";
    	arg2[1] = "C:/Documents and Settings/desenvolvimento/exe/balanced.spa";
    	arg2[2] = "-i";
    	arg2[3] = "C:/Documents and Settings/desenvolvimento/exe/balanced.in";
    	
    	
        new Runner().run(arg2,null);

        /*
			 // The simplest way to run a virtual machine from another java program
			 // is ilustrated bellow:

             String defPars[] = { "-m","cc\\sintatico.spa","-i","cc\\lista.in","-o","lista.spa","-im","ccl\\lexico.spa\n" };
             new Runner().run(defPars);
        */

    }
}
